# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_utils.ipynb.

# %% auto 0
__all__ = ['yaml2dict']

# %% ../nbs/01_utils.ipynb 4
import yaml

# %% ../nbs/01_utils.ipynb 5
def yaml2dict(file_path):
    with open(file_path, "r") as file:
        data = yaml.safe_load(file)
    return data
