# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/toolformer',
                'doc_host': 'https://xrsrke.github.io',
                'git_url': 'https://github.com/xrsrke/toolformer',
                'lib_path': 'toolformer'},
  'syms': { 'toolformer.api': { 'toolformer.api.BaseAPI': ('api.html#baseapi', 'toolformer/api.py'),
                                'toolformer.api.BaseAPI.__call__': ('api.html#baseapi.__call__', 'toolformer/api.py'),
                                'toolformer.api.BaseAPI.__init__': ('api.html#baseapi.__init__', 'toolformer/api.py'),
                                'toolformer.api.BaseAPI.execute': ('api.html#baseapi.execute', 'toolformer/api.py'),
                                'toolformer.api.CalculatorAPI': ('api.html#calculatorapi', 'toolformer/api.py'),
                                'toolformer.api.CalculatorAPI.execute': ('api.html#calculatorapi.execute', 'toolformer/api.py'),
                                'toolformer.api.WolframeAPI': ('api.html#wolframeapi', 'toolformer/api.py'),
                                'toolformer.api.WolframeAPI.execute': ('api.html#wolframeapi.execute', 'toolformer/api.py')},
            'toolformer.data_generator': { 'toolformer.data_generator.AugmentedCandidate': ( 'data_generator.html#augmentedcandidate',
                                                                                             'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator': ( 'data_generator.html#datagenerator',
                                                                                        'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator.__init__': ( 'data_generator.html#datagenerator.__init__',
                                                                                                 'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator._calculate_loss': ( 'data_generator.html#datagenerator._calculate_loss',
                                                                                                        'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator._calculate_weighted_loss': ( 'data_generator.html#datagenerator._calculate_weighted_loss',
                                                                                                                 'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator._compute_weight': ( 'data_generator.html#datagenerator._compute_weight',
                                                                                                        'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator._filter_candidate_by_threshold': ( 'data_generator.html#datagenerator._filter_candidate_by_threshold',
                                                                                                                       'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator._generate_conditioning_prompts': ( 'data_generator.html#datagenerator._generate_conditioning_prompts',
                                                                                                                       'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator._normalize_weights': ( 'data_generator.html#datagenerator._normalize_weights',
                                                                                                           'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator.extract_api_request_content': ( 'data_generator.html#datagenerator.extract_api_request_content',
                                                                                                                    'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator.extract_api_syntax': ( 'data_generator.html#datagenerator.extract_api_syntax',
                                                                                                           'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator.filter_api': ( 'data_generator.html#datagenerator.filter_api',
                                                                                                   'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator.generate': ( 'data_generator.html#datagenerator.generate',
                                                                                                 'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator.obtain_api_response': ( 'data_generator.html#datagenerator.obtain_api_response',
                                                                                                            'toolformer/data_generator.py'),
                                           'toolformer.data_generator.DataGenerator.sample_api_position': ( 'data_generator.html#datagenerator.sample_api_position',
                                                                                                            'toolformer/data_generator.py')},
            'toolformer.model': {'toolformer.model.Model': ('model.html#model', 'toolformer/model.py')},
            'toolformer.prompt': {},
            'toolformer.utils': {'toolformer.utils.yaml2dict': ('utils.html#yaml2dict', 'toolformer/utils.py')}}}
